"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppJoinBuilder = void 0;
var ReflectAdaptor_1 = require("../decorators/base/ReflectAdaptor");
var MapUtil_1 = require("../utils/MapUtil");
var get_app_join_function_1 = require("./internal/get_app_join_function");
/**
 * Application level join builder.
 *
 * `AppJoinBuilder` is a helper class who can perform the application level join very
 * easily. With this `AppJoinBuilder`, you don't need to implement the application level
 * joining script, with the {@link HashMap}, by yourself. The joining would be done by
 * this `AppJoinBuilder` automatically.
 *
 * Also, you can build such `AppJoinBuilder` safely and conveniently. When you are
 * building up an `AppJoinBuilder` instance from a specific ORM class, the
 * `AppJoinBuilder` would analyze relationships of the target ORM class in the
 * compilation level. Therefore, if you take any mistake, it would be detected in the
 * IDE directly. Besides, when you're finding neighbor entities to join,
 * `AppJoinBuilder` will hint you through the auto-completion.
 *
 * ![Hints by the Auto Completion](https://raw.githubusercontent.com/samchon/safe-typeorm/master/assets/demonstrations/app-join-builder.gif)
 *
 * Furthermore, interface of the `AppJoinBuilder` is exactly same with the
 * {@link JoinQueryBuilder}, another helper class who can implement DB join very
 * conveniently and safely. Therefore, you can convert between `AppJoinBuilder` and
 * {@link JoinQueryBuilder} in anytime. Feel free to selecting one of them and convert
 * them whenever you want.
 *
 * On the other hand, if you're performing this application level join only for the
 * JSON data construction, {@link JsonSelectBuilder} would be much suitable. In
 * {@link JsonSelectBuilder} also utilizes the application level joining and it even
 * automates the JSON interface definition and data conversion in the compilation level.
 *
 * ## What Application Join is:
 * The application level join means that joining related records are done not by the
 * DB join query, but through the manual application code using the {@link HashMap}
 * with their primary and foreign column values.
 *
 * Comparing those DB join query and application level joining, the application level
 * joining consumes much fewer resources and its elapsed time is also much shorter than
 * the DB join query. Those differences grow up whenever the join relationship be more
 * complicate.
 *
 * Type         | DB Join   | App Join
 * :------------|----------:|----------:
 * Records      | 2,258,000 | 165
 * Elapsed Time | 8.07508   | 0.00262
 *
 * @template Mine Target entity to perform the app join
 * @reference [stackoverflow/join-queries-vs-multiple-queries](https://stackoverflow.com/questions/1067016/join-queries-vs-multiple-queries)
 * @author Jeongho Nam - https://github.com/samchon
 */
var AppJoinBuilder = /** @class */ (function () {
    /* -----------------------------------------------------------
        CONSTRUCTORS
    ----------------------------------------------------------- */
    /**
     * Default Constructor.
     *
     * @param mine Target ORM class to perform the app joining
     */
    function AppJoinBuilder(mine) {
        this.mine_ = mine;
        this.children_ = new Map();
    }
    /**
     * Configure a neighbor entity to join.
     *
     * `AppJoinBuilder.join()` is a method configuring a neighbor entity to join, by
     * specifying an accessor field of the *Mine* class, who've defined the
     * relationship decorator with the target neighbor entity class.
     *
     * You also can configure another app joining targets of the target neighbor
     * entity by writing *closure* function or calling additional methods to the
     * returned `AppJoinBuilder` instance for the target neighbor entity. Of course,
     * such app join definition chaining would also benefic from the compilation type
     * checking and auto-completion hints, too.
     *
     * If you've configured duplicated neighbor entity to join again, it will not affect
     * to the app joining target. However, the *closure* function would be affected.
     * Also, calling additional methods to the returned `AppJoinBuilder` instance for
     * the target neighbor entity would be affected, too.
     *
     * @param target Field of *Mine* who've defined the relationship decorator to join or tuple with filter function
     * @param closure Closure function for additional app joins for the target entity
     * @returns Emplaced `AppJoinBuilder` instance for the target entity
     */
    AppJoinBuilder.prototype.join = function (target, closure) {
        var _this = this;
        var field = target instanceof Array ? target[0] : target;
        var filter = target instanceof Array ? target[1] : null;
        // PREPARE BUILDER
        var child = MapUtil_1.MapUtil.take(this.children_, field, function () {
            var metadata = ReflectAdaptor_1.ReflectAdaptor.get(_this.mine_.prototype, field);
            var builder = new AppJoinBuilder(metadata.target());
            return { metadata: metadata, builder: builder, filter: filter };
        });
        // CALL CLOSURE
        if (closure !== undefined)
            closure(child.builder);
        // RETURNS BUILDER
        return child.builder;
    };
    /**
     * Safe factory method for the `AppJoinBuilder`.
     *
     * ## Unintended Mistake
     * When creating an `AppJoinBuilder` instance and configuring neighbor entities
     * to be joined through the {@link AppJoinBuilder.join} method, you can take a
     * mistake that omitting a specific neighbor entity.
     *
     * If you take such mistake, there would not be any compile-time and run-time error
     * would be occurred. However, unintended SELECT statements would be repeatedly
     * queried whenever you access to the specific relationship decorated member what
     * you've omitted.
     *
     * Therefore, if you take such mistake, you may failed to performance tuning
     * through the application level joining plan.
     *
     * ## Safe Initializer
     * To avoid such unintended mistake, you can use this factory method
     * `AppJoinBuilder.initialize` with the {@link AppJoinBuilder.Initialized} typed
     * object, instead of creating the `AppJoinBuilder` instance by yourself and
     * configuring manual app joins through the {@link AppJoinBuilder.join} method.
     *
     * The {@link AppJoinBuilder.Initialized} type enforces you to list up all of the
     * relationship decorated fields. Even if you don't want to join with a specific
     * neighbor entity, the {@link AppJoinBuilder.Initialized} type compels you to
     * assign the `undefined` value on the matched relationship decorated field.
     *
     *   - Assignable properties
     *     - `undefined`: do not join with the neighbor
     *     - `"join"`: join with the neighbor, but no cascade app joining more
     *     - `AppJoinBuilder<Neighbor> instance`: join with the neighbor and continue cascade app joining through a `AppJoinBuilder` instance
     *     - {@link AppJoinBuilder.Closure}: join with the neighbor and continue cascade app joining through a closure function
     *
     * @param creator Target ORM class to perform the app joining
     * @param input List of relationship decorated fields with joining plan
     * @returns Newly created `AppJoinBuilder` instance
     */
    AppJoinBuilder.initialize = function (creator, input) {
        var e_1, _a;
        var builder = new AppJoinBuilder(creator);
        try {
            for (var _b = __values(Object.entries(input)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), label = _d[0], data = _d[1];
                if (!data)
                    continue;
                var key = label;
                var filter = data instanceof Array ? data[1] : null;
                var value = data instanceof Array ? data[0] : data;
                if (value instanceof AppJoinBuilder)
                    if (filter !== null)
                        builder.set(key, filter, value);
                    else
                        builder.set(key, value);
                else if (value === "join")
                    if (filter !== null)
                        builder.join([key, filter]);
                    else
                        builder.join(key);
                else if (typeof value === "function")
                    if (filter !== null)
                        builder.join([key, filter], value);
                    else
                        builder.join(key, value);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return builder;
    };
    /**
     * Execute the application join.
     *
     * `AppJoinBuilder.execute()` is the final execution method performing the
     * application level joining, who've defined by {@link AppJoinBuilder.join} or
     * {@link AppJoinBuilder.initialize}, to the *data*. Therefore after the execution,
     * the *data* record(s) would be linked to the specified neighbor entity records.
     *
     * However, you should know that, this method can't predicate whether the execution
     * on the *data* is duplicated or not. Thus, if you've changed some properties of
     * a linked entity record after the execution and call this execution method again,
     * you may lose the modified property values.
     *
     * @param data Target record(s) to be joined
     */
    AppJoinBuilder.prototype.execute = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(data instanceof Array))
                            data = [data];
                        return [4 /*yield*/, this._Execute(data)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    AppJoinBuilder.prototype._Execute = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, _c, field, child, func, output, e_2_1;
            var e_2, _d;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _e.trys.push([0, 6, 7, 8]);
                        _a = __values(this.children_.entries()), _b = _a.next();
                        _e.label = 1;
                    case 1:
                        if (!!_b.done) return [3 /*break*/, 5];
                        _c = __read(_b.value, 2), field = _c[0], child = _c[1];
                        func = (0, get_app_join_function_1.get_app_join_function)(child.metadata.type);
                        return [4 /*yield*/, func(this.mine_, child.metadata, data, field, {
                                filter: child.filter || null,
                                targetData: null,
                                routerData: null,
                            })];
                    case 2:
                        output = _e.sent();
                        if (!(output.length !== 0)) return [3 /*break*/, 4];
                        return [4 /*yield*/, child.builder._Execute(output)];
                    case 3:
                        _e.sent();
                        _e.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 1];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_2_1 = _e.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    Object.defineProperty(AppJoinBuilder.prototype, "size", {
        /* -----------------------------------------------------------
            ACCESSORS
        ----------------------------------------------------------- */
        /**
         * Number of neighbor entities to be joined.
         */
        get: function () {
            return this.children_.size;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Iterator of relationship decorator fields to be joined.
     */
    AppJoinBuilder.prototype.keys = function () {
        return this.children_.keys();
    };
    /**
     * Iterator of neighbor entities to be joined.
     */
    AppJoinBuilder.prototype.values = function () {
        return new ValueIterator(this.children_.values());
    };
    /**
     * Iterator of key-value pairs to be joined.
     */
    AppJoinBuilder.prototype.entries = function () {
        return new EntryIterator(this.children_.entries());
    };
    /**
     * For of iterator of entries to be joined.
     */
    AppJoinBuilder.prototype[Symbol.iterator] = function () {
        return this.entries();
    };
    /**
     * Loop value-key pairs to be joined.
     *
     * @param closure A function accepting value-key pairs for every elements
     */
    AppJoinBuilder.prototype.forEach = function (closure) {
        var e_3, _a;
        try {
            for (var _b = __values(this), _c = _b.next(); !_c.done; _c = _b.next()) {
                var tuple = _c.value;
                closure(tuple[1], tuple[0], this);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    /* -----------------------------------------------------------
        ELEMENTS I/O
    ----------------------------------------------------------- */
    /**
     * Test whether a specific neighbor entity has been configured.
     *
     * @param key Field of *Mine* who've defined the relationship decorator to join
     * @returns Whether the specified neighbor entity has been configured or not
     */
    AppJoinBuilder.prototype.has = function (key) {
        return this.children_.has(key);
    };
    /**
     * Get an `AppJoinBuilder` instance of a specific neighbor entity.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @returns `AppJoinBuilder` instance of the neighbor entity or `undefined`
     */
    AppJoinBuilder.prototype.get = function (field) {
        var child = this.children_.get(field);
        return child !== undefined ? child.builder : undefined;
    };
    AppJoinBuilder.prototype.set = function (field) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var metadata = ReflectAdaptor_1.ReflectAdaptor.get(this.mine_.prototype, field);
        var builder = args.length === 1 ? args[0] : args[1];
        var filter = args.length === 1 ? null : args[0];
        this.children_.set(field, {
            metadata: metadata,
            builder: builder,
            filter: filter,
        });
        return this;
    };
    /**
     * Remove a specific neighbor entity to be joined.
     *
     * @param field Field of *Mine* who've defined the relationship decorator to join
     * @returns Whether the specified neighbor has been configured and succeeded to remove it or not
     */
    AppJoinBuilder.prototype.delete = function (field) {
        return this.children_.delete(field);
    };
    return AppJoinBuilder;
}());
exports.AppJoinBuilder = AppJoinBuilder;
(function (AppJoinBuilder) {
    /* eslint-disable */
    /**
     * Maximum variable count of the parameter binding in one SQL query.
     */
    AppJoinBuilder.MAX_VARIABLE_COUNT = 32700;
})(AppJoinBuilder = exports.AppJoinBuilder || (exports.AppJoinBuilder = {}));
exports.AppJoinBuilder = AppJoinBuilder;
/* -----------------------------------------------------------
    ITERATORS
----------------------------------------------------------- */
/**
 * @internal
 */
var ValueIterator = /** @class */ (function () {
    function ValueIterator(values_) {
        this.values_ = values_;
    }
    ValueIterator.prototype.next = function () {
        var it = this.values_.next();
        if (it.done === true)
            return it;
        return {
            done: false,
            value: it.value.builder,
        };
    };
    ValueIterator.prototype[Symbol.iterator] = function () {
        return this;
    };
    return ValueIterator;
}());
/**
 * @internal
 */
var EntryIterator = /** @class */ (function () {
    function EntryIterator(entries_) {
        this.entries_ = entries_;
    }
    EntryIterator.prototype.next = function () {
        var it = this.entries_.next();
        if (it.done === true)
            return it;
        return {
            done: false,
            value: [it.value[0], it.value[1].builder],
        };
    };
    EntryIterator.prototype[Symbol.iterator] = function () {
        return this;
    };
    return EntryIterator;
}());
//# sourceMappingURL=AppJoinBuilder.js.map