import * as orm from "typeorm";
import { Belongs } from "../decorators/Belongs";
import { Has } from "../decorators/Has";
import { Creator } from "../typings/Creator";
import { OmitNever } from "../typings/OmitNever";
import { PrimaryColumnType } from "../typings/PrimaryColumnType";
import { Primitive } from "../typings/Primitive";
import { Relationship } from "../typings/Relationship";
import { Same } from "../typings/Same";
import { DEFAULT } from "../DEFAULT";
/**
 * JSON Select Builder.
 *
 * @template Mine Target entity to convert to JSON data
 * @template InputT Type of input listing up the joining plans
 * @template Destination Output JSON type. Default is {@link JsonSelectBuilder.Output}
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare class JsonSelectBuilder<Mine extends object, InputT extends JsonSelectBuilder.Input<Mine>, Destination = JsonSelectBuilder.Output<Mine, InputT>> {
    readonly input: Readonly<InputT>;
    readonly mapper?: JsonSelectBuilder.Output.Mapper<Mine, InputT, Destination> | undefined;
    private readonly mine_;
    private readonly joiner_;
    /**
     * Default Constructor.
     *
     * @param mine Target ORM class to perform the DB join
     * @param input List of relationship decorated fields with joining plan
     * @param mapper Map function to convert output type
     */
    constructor(mine: Creator<Mine>, input: Readonly<InputT>, mapper?: JsonSelectBuilder.Output.Mapper<Mine, InputT, Destination> | undefined);
    /**
     * Execute app join.
     *
     * @param data Target record(s) to be joined
     */
    join(data: Mine | Mine[]): Promise<void>;
    /**
     *
     * @param record
     * @param skipAppJoin
     * @returns
     */
    getOne(record: Mine, skipAppJoin?: boolean): Promise<Destination>;
    /**
     *
     * @param records
     * @param skipAppJoin
     * @returns
     */
    getMany(records: Mine[], skipAppJoin?: boolean): Promise<Destination[]>;
    private _Convert;
    private _Convert_Recursive;
}
export declare namespace JsonSelectBuilder {
    /**
     *
     */
    export type Input<Mine extends object> = Partial<OmitNever<{
        [P in keyof Mine]: Mine[P] extends Relationship<infer Target> ? Mine[P] extends BelongsCommon<Target, any, infer TargetOptions> ? TargetOptions extends {
            nullable: true;
        } ? Same<Mine, Target> extends true ? "recursive" | Tuplize<Target, "join"> | DEFAULT | undefined : Tuplize<Target, JsonSelectBuilder<Target, any, any>> | Tuplize<Target, "join"> | DEFAULT | undefined : Same<Mine, Target> extends true ? "recursive" | Tuplize<Target, "join"> | DEFAULT | undefined : Tuplize<Target, JsonSelectBuilder<Target, any, any>> | Tuplize<Target, "join"> | DEFAULT | undefined : Same<Mine, Target> extends true ? "recursive" | Tuplize<Target, "join"> | undefined : Tuplize<Target, JsonSelectBuilder<Target, any, any>> | Tuplize<Target, "join"> | undefined : never;
    }>>;
    /**
     *
     */
    export type Output<Mine extends object, InputT extends object> = Primitive<Mine> & OmitNever<{
        [P in keyof (Mine | InputT)]: Escape<InputT[P]> extends JsonSelectBuilder<infer Target, any, infer Destination> ? Mine[P] extends BelongsCommon<Target, any, infer Options> ? Options extends {
            nullable: true;
        } ? Destination | null : Destination : Mine[P] extends HasOneCommon<Target, infer Ensure> ? Ensure extends true ? Destination : Destination | null : Destination[] : Escape<InputT[P]> extends "recursive" ? Mine[P] extends BelongsCommon<Mine, any, infer Options> ? Options extends {
            nullable: true;
        } ? Output.RecursiveReference<Mine, P> | null : never : Mine[P] extends HasOneCommon<Mine, infer Ensure> ? Ensure extends true ? never : Output.RecursiveReference<Mine, P> | null : Mine[P] extends HasManyCommon<Mine> ? Output.RecursiveArray<Mine, P> : Mine[P] extends Has.ManyToMany<Mine, any> ? Output.RecursiveArray<Mine, P> : never : InputT[P] extends DEFAULT ? Mine[P] extends BelongsCommon<any, infer PrimaryType, infer Options> ? Options extends {
            nullable: true;
        } ? PrimaryColumnType.ValueType<PrimaryType> | null : PrimaryColumnType.ValueType<PrimaryType> : never : Escape<InputT[P]> extends "join" ? Mine[P] extends BelongsCommon<infer Target, any, infer Options> ? Options extends {
            nullable: true;
        } ? Primitive<Target> | null : Primitive<Target> : Mine[P] extends HasOneCommon<infer Target, infer Ensure> ? Ensure extends true ? Primitive<Target> : Primitive<Target> | null : Mine[P] extends HasManyCommon<infer Target> ? Primitive<Target>[] : never : never;
    }>;
    export namespace Output {
        /**
         *
         */
        type RecursiveReference<Mine extends object, Key extends keyof Mine> = Primitive<Mine> & {
            [P in Key]: RecursiveReference<Mine, Key> | null;
        };
        /**
         *
         */
        type RecursiveArray<Mine extends object, Key extends keyof Mine> = Primitive<Mine> & {
            [P in Key]: RecursiveArray<Mine, Key>[];
        };
        /**
         *
         */
        interface Mapper<Mine extends object, InputT extends Input<Mine>, Destination> {
            (output: Output<Mine, InputT>, model: Mine): Destination;
        }
    }
    type BelongsCommon<Target extends object, PrimaryKey extends PrimaryColumnType, Options extends {
        nullable: boolean;
    }> = Belongs.ManyToOne<Target, PrimaryKey, Options> | Belongs.OneToOne<Target, PrimaryKey, Options> | Belongs.External.OneToOne<Target, PrimaryKey, Options> | Belongs.External.ManyToOne<Target, PrimaryKey, Options>;
    type HasOneCommon<Target extends object, Ensure extends boolean> = Has.OneToOne<Target, Ensure> | Has.External.OneToOne<Target, Ensure>;
    type HasManyCommon<Target extends object> = Has.OneToMany<Target> | Has.External.OneToMany<Target> | Has.ManyToMany<Target, any>;
    type Tuplize<Target extends object, Value> = Value | [Value, (stmt: orm.SelectQueryBuilder<Target>) => void];
    type Escape<T> = T extends [infer F, Function] ? F : T;
    export {};
}
