"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteTimestampColumn = void 0;
var typeorm_1 = require("typeorm");
/**
 * Deletion timstamp column for `postgres`.
 *
 * When defining datetime column, time zone is very important when considering global
 * service. However, default `DeleteDateColumn` of `typeorm` does not archive the time zone
 * when using `postgres` database. Furthermore, `typeorm` does not archive milliseconds.
 *
 * I think it's a critical mistake of `typeorm`, but they think it's not a bug but spec.
 *
 * Therefore, `safe-typeorm` supports custom deletion timestamp column which supports time
 * zone and milliseconds. When using `postgres` database, use this `DeleteTimestampColumn`
 * instead of the `DeleteDateColumn`.
 *
 * @param options Additional options if required
 * @returns Property decorator function
 */
var DeleteTimestampColumn = function (options) {
    return (0, typeorm_1.DeleteDateColumn)(__assign(__assign({}, (options || {})), { precision: 3, type: "timestamp with time zone" }));
};
exports.DeleteTimestampColumn = DeleteTimestampColumn;
//# sourceMappingURL=DeleteTimestampColumn.js.map