import * as orm from "typeorm";
import { Comparator } from "../../typings/Comparator";
import { Creator } from "../../typings/Creator";
import { BelongsManyToOne } from "./BelongsManyToOne";
import { BelongsOneToOne } from "./BelongsOneToOne";
/**
 * Type for a variable using the `Has.ManyToMany` decorator.
 *
 * @template Target Type of the target model, who has the N: N relationship with this model
 *                  class *Mine*, through the *Route* model class
 * @tempalte Router Type of the router model, who intermediates the M: N relationship.
 */
export declare type HasManyToMany<Target extends object, Router extends object> = HasManyToMany.Accessor<Target, Router>;
/**
 * Decorator function for the "N:N has" relationship.
 *
 * @template Mine Type of this model class who is using it, the `Has.ManyToMany` decorator
 * @template Target Type of the target model class, who has the N: N relationship with this
 *                  model class *Mine*, through the *Route* model class
 * @template Router Type of the router model who intermediates those *Mine* and *Route* model
 *                  classes to resolve the N: N relationship
 * @param target A closure function returning the *Target* model class, who has the N: N
 *                  relationship with this model class *Mine* through the *Route* model class
 * @param router A closure function returning the *Router* model class, who intermediates
 *                  those *Mine* and *Route* model classes to resolve the N: N relationship
 * @param targetInverse A closure function returning the {@link Belongs.ManyToOne} typed member
 *                      variable, who is pointing this model class *Mine*, from the *Route*
 *                      model class
 * @param myInverse A closure function returning the {@link Belongs.ManyToOne} typed member
 *                  variable, who is poining the target model class *Target*, from the *Rote*
 *                  mode class
 * @param comparator Comparator for sorting if required.
 * @return The *PropertyDecorator* function
 */
export declare function HasManyToMany<Mine extends object, Target extends object, Router extends object>(target: Creator.Generator<Target>, router: Creator.Generator<Router>, targetInverse: (router: Router) => BelongsManyToOne<Target, any> | BelongsOneToOne<Target, any>, myInverse: (router: Router) => BelongsManyToOne<Mine, any> | BelongsOneToOne<Target, any>, comparator?: Comparator<HasManyToMany.ITuple<Target, Router>>): PropertyDecorator;
export declare namespace HasManyToMany {
    interface ITuple<Target extends object, Router extends object> {
        target: Target;
        router: Router;
    }
    class Accessor<Target extends object, Router extends object> {
        private readonly metadata_;
        private readonly mine_;
        private readonly data_;
        private constructor();
        get(): Promise<Target[]>;
        set(value: Target[]): Promise<void>;
        statement(joinAndSelect?: boolean): orm.SelectQueryBuilder<Router>;
        private _Get;
    }
}
