"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getColumn = void 0;
var get_column_name_tuple_1 = require("./internal/get_column_name_tuple");
/**
 * Get column name.
 *
 * `getColumn()` is a global function returning the column name.
 *
 * Unlike writing the column name by yourself manually who can result in the critical runtime
 * error by a typing error, the `getColumn()` can detect the typing error in the compilation level.
 *
 * The `getColumn()` even supports the table alias, therefore the table alias addicted column name
 * also can take advantage of the compile time validation. In such reason, if you don't ignore
 * error message from the TypeScript compiler, there can't be any runtime error that is caused by
 * the mis-typing error in the SQL column specification level.
 *
 * @template T Type of a model class
 * @template Literal Type of a literal who represents the field that is defined in the *T* model
 * @param fieldLike Name of the target field in the model class. The field name can contain the
 *                  table alias.
 * @param alias Alias of the target column
 * @return The exact column name who never can be the runtime error
 */
function getColumn(creator, fieldLike, alias) {
    var tuple = (0, get_column_name_tuple_1.get_column_name_tuple)(creator, fieldLike);
    if (alias === undefined)
        alias = tuple[1];
    var target = tuple[0] ? "".concat(tuple[0], ".").concat(tuple[1]) : tuple[1];
    return alias === null ? target : "".concat(target, " AS ").concat(alias);
}
exports.getColumn = getColumn;
//# sourceMappingURL=getColumn.js.map