/**
 * Password component for embedding.
 *
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare class Password {
    /**
     * The password.
     */
    private password;
    isNull(): boolean;
    /**
     * Configure password.
     *
     * Configure the password with one-way encryption. Therefore, no one can know the
     * original password, even if the one is a system administrator.
     *
     * @param str The password to configure.
     */
    set(str: string | null): Promise<void>;
    /**
     * Test whether the password is matched with the original.
     *
     * The password configured in this record has been one-way encrypted by `bcrypt`
     * algorithm. Therefore, no one can know the original password. Therefore, just
     * only testing the equality through this {@link equals}() is possible.
     *
     * @param str Target to be compared.
     * @return Whether equal or not.
     */
    equals(str: string | null): Promise<boolean>;
}
